      subroutine calcmatrix (i,j,lconv)
      
C     This subroutine calculates the matrix describing convective 
C     redistribution of mass in a grid column, using the subroutine
C     convect.f provided by Kerry Emanuel.
C     Petra Seibert, Bernd C. Krueger, 2000-2001
C     Update April 2001 (iterative corrections)

C  delt ..      time step for convection [s]
C  delt0 ..     initial time step for convection [s]
C  time ..       elapsed time in convection loop [s]
C  endtime ..    end time in convection loop
C  timem1,timem2 begin and end of time interval in which convection would
C                act upon the tracer for midlatitudes
C  timet1,timet2 the same for the tropics
C  phitm ..      latitude that separates tropical from midlatitude regime
C                in convection (same value used for NH and SH - might be
C                made flexible in the future)
C  nitermax ..   maximum number of iterations for mass correction
C                typical values 100..1000
C  dmtotmax ..   maximum tolerable value of mean relative mass deficit
C                typical values 1e-4..1e-7
C  iteration criteria must be the stricter the longer the run will go on.

      include 'includepar'
      include 'includecom'
      include 'includeconv'

      integer i, j, niter, nitermax
      integer iconvect, ishort, iflag, k, kk, kuvz, nf1, nf2
     
C     1-d variables for convection
      real  fft(nconvlevmax),ffq(nconvlevmax)
C     variables for redistribution matrix
      real cbmf, cbmfold, precip, qprime
      real time, tprime, wd, f_qvsat, dwftime,dwffact
      real delt, delt0, endtime, timem1, timet1, timem2, timet2, phitm
      real time1,time2, dk, dkk, dmtot, dmtotold, dmtotmax
      real d1,d1old,d1max1,d1max2
      real csumk(nconvlevmax), rk(nconvlevmax)
      real csumkk(ntracermax), rkk(ntracermax)        
      logical lconv
      parameter (delt0=1200.)
C       inital time step is 20 minutes. This is a sensitive quantity!
C       Don't change without good reason and testing.   
      parameter (timem1= 6.*delt0, timem2= 8.*delt0)
      parameter (timet1=11*delt0, timet2=15.*delt0)
      parameter (phitm=35., nitermax=13, dmtotmax=1.e-4)
      parameter (d1max1=0.01,d1max2=1.e-6)
      if (abs(ylat0+j*dy) .gt. phitm) then
C       midlatitudes
        time1 = timem1
        time2 = timem2
        endtime = timem2
      else
C       tropics
        time1 = timet1
        time2 = timet2
        endtime = timet2
      endif

      lconv = .false.

C     assign time indices - note that calculation of convection must
C     be done forward in the true time both in fwd or bwd calculations!

      if (ldirect .eq. 1) then
        nf1 = memind(1)
        nf2 = memind(2)
        dwftime = memtime(2) - memtime(1)
      else
        nf2 = memind(1)
        nf1 = memind(2)
        dwftime = memtime(1) - memtime(2)
      endif
    
      
C calculate pressure at eta levels for use in convect
C and assign temp & spec. hum. to 1D workspace
C -------------------------------------------------------

C plev(1) is the pressure at the first level above ground
C phlev(k) is the pressure between levels k-1 and k 
C dp(k) is the pressure difference "around" ctralev(k)
C phlev(kmax) must also be defined 1/2 level above plev(kmax)
C Therefore, we define k = kuvz-1 and let kuvz start from 2
C top layer cannot be used for convection because p at top of this layer is
C not given


      phlev(1) = ps(i,j,1,nf1)/100.
C Emanuel subroutine needs pressure in hPa, therefore convert all pressures       
      do kuvz = 2,nuvz
        k = kuvz-1
        plev(k) = (akz(kuvz) + bkz(kuvz)*ps(i,j,1,nf1))/100.
        phlev(kuvz) = (akm(kuvz) + bkm(kuvz)*ps(i,j,1,nf1))/100.
        dp(k) = phlev(k) - phlev(kuvz)

	tlev(k) = tth(i,j,kuvz,nf1)
        qlev(k) =  qvh(i,j,kuvz,nf1) 


        qslev(k) = f_qvsat( plev(k)*100., tlev(k) )
C       fft,ffq are tendencies from ECMWF data        

	fft(k) = ( tth(i,j,kuvz,nf2) - tth(i,j,kuvz,nf1) ) / dwftime
        ffq(k) = ( qvh(i,j,kuvz,nf2) - qvh(i,j,kuvz,nf1) ) / dwftime


C       Initialize tracer in displacement matrix with delta function
C       Total tracer ctm0 is in mass/m2 units and contained in one layer.
C       Conversion mixing ratio (needed in convect) <-> mass:
C       $\chi = g ctm0 / dp$, where g = 9.81 m/s2 and dp is the pressure
C       difference across the layer. We ignore the constant g in both
C       directions.
        do kk=1,ntracer
          ctralev(k,kk)=0.  
        enddo
        ctralev(k,k) = ctm0
      enddo

C     further initializations (cloud-base mass flux cbmf)
      cbmf = 0.
C     note that Emanuel says it is important
C     a. to set this =0. every grid point
C     b. to keep this value in the calling programme in the iteration

c     time in sec for subsequent calls of convect
      time=0.
      ishort = 1

c CONVECTION LOOP===================================================
      do 100 iconvect=1,1000 ! upper bound for safety only, no physical meaning

90      continue
        delt = delt0 / float(ishort)
        time = time + delt
        cbmfold = cbmf
        call CONVECT
     &           (NCONVLEVMAX, NCONVLEV, NTRACER, DELT, IFLAG,
     &            PRECIP, WD, TPRIME, QPRIME, CBMF)
     
c       skip loop if no convection
        if (iflag .ne. 1 .and. iflag .ne. 4) goto 200

c       test for CFL criterion and reduce time step if required    
        if (iflag.eq.4 .and. ishort.lt.16) then
          ishort = ishort*2
          cbmf = cbmfold
          time = time - delt
          goto 90
        endif
    
c       Update variables

        do 60 k=1,nconvlev
          tlev(k) = tlev(k) + delt*(FT(k) + fft(k))
          qlev(k) = qlev(k) + delt*(FQ(k) + ffq(k))
          qslev(k) = f_qvsat( plev(k)*100., tlev(k) )
          if (time1 .le. time .and. time .le. time2) then
	  
            do kk=1,ntracer
              ctralev(k,kk) = abs(ctralev(k,kk)+delt*ftra(k,kk))
             if (ftra(k,kk) .ne. 0.) lconv = .true.
            enddo
          endif
60      continue

C       terminate the convection loop after a prescribed time
C       according to the time interval that is used to calculate
C       ctralev, we can leave after endtime. 

        if (nint(time) .ge. endtime) then 
          goto 200
        endif  

100   continue
c END CONVECTION LOOP===================================================

200   continue

      if (lconv) then
C     Now check and correct iteratively for negative values, mass conservation
C     per tracer species, and mass conservation for each arrival level
C     Needed only if there has been convection.
       
C     initialise iteration
      dmtotold=1.e24
      d1old=1.e24
      niter = 0
C     scale convective tendencies depending on the time between the two
C     wind fields
      dwffact = dwftime/(3600.*6.) 


C     go to mass units 
        do kk=1,ntracer
          do k =1,nconvlev
            ctralev(k,kk) = ctralev(k,kk)*dp(k)*dwffact
            if (ctralev(k,kk).lt.0) ctralev(k,kk) = -ctralev(k,kk)
          enddo
        enddo
        
C       begin iteration        
70      continue

          niter=niter+1        
          dmtot=0.
          d1 = 0.
        
C         calculate sums over rows and columns and determine misfits
          do k=1,nconvlev
            csumk(k)=0.
          enddo
        
          do 82 k=1,nconvlev
82          csumk(k)=0.       
          do 80 kk=1,ntracer
            csumkk(kk)=0.
            do k=1,nconvlev
              csumkk(kk) = csumkk(kk) + ctralev(k,kk)
              csumk(k) = csumk(k) + ctralev(k,kk)
            enddo
            rkk(kk) = ctm0*dp(kk) - csumkk(kk) 
80        continue

          do k=1,nconvlev
            rk(k) = ctm0*dp(k) - csumk(k)
          enddo

C         do the correction
          do kk=1,ntracer
            do k =1,nconvlev
              dk = rk(k)/(ctm0*dp(k))
              dkk = rkk(kk)/(ctm0*dp(kk))
              dmtot=dmtot + dk*dk + dkk*dkk
              if (k.eq.1) d1 = d1 + dk*dk
              dmtot=dmtot + dk*dk + dkk*dkk
              if (1.+dk+dkk .lt. 0) then
                ctralev(k,kk) = 0.
              else
                ctralev(k,kk) = ctralev(k,kk) * ( 1. + dk + dkk )
              endif
            enddo
          enddo
 
C         check for lack of convergence
          if (dmtot .gt. dmtotold .and. d1.gt.d1old) then
            if (d1 .gt. d1max1) lconv = .false.
            goto 900
          endif
          dmtotold = dmtot	 
          d1old=d1
     
       if (niter .le. nitermax .and.
     &        (dmtot/(ntracer+nconvlev) .ge. dmtotmax
     &     .or. d1 .ge. d1max2 ) ) goto 70  

C      go back to mixing ratios
       do kk=1,ntracer
         do k =1,nconvlev
           ctralev(k,kk) = ctralev(k,kk)/dp(k)
         enddo
       enddo

       endif
C      end convection
	
900   continue


      end
